/*=============================================================================*
 *  Copyright 2004 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.ws.eventing;

import org.apache.commons.collections.map.ReferenceMap;
import org.apache.ws.resource.impl.AbstractResourceHome;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

/**
 * This class should not be modified.
 * <p/>
 * The class acts as an extension of the AbstractResourceHome which
 * maintains a reference to the STATIC map needed for accessing resources via JNDI.
 * <p/>
 * This ensures the resources will be accessible if serialized via JNDI in certain platforms.
 */
abstract class AbstractSubscriptionHome
   extends AbstractResourceHome
{
   /**
    * The static reference which is maintained and used by the AbstractResourceHome.
    * <p/>
    * Do not modify.
    */
   private static Map m_resource;

   /**
    * Do not modify this method.
    */
   protected final Map initResourceMap(  )
   throws NamingException
   {
      Context initialContext = new InitialContext(  );
      if ( m_resourceIsPersistent )
      {
         m_resources = new ReferenceMap( ReferenceMap.HARD, ReferenceMap.SOFT, true );
         initCachePolicy( initialContext );
      }
      else
      {
         m_resources = new HashMap(  );
      }

      m_resources = Collections.synchronizedMap( m_resources );
      return m_resources;
   }
}