/*=============================================================================*
 *  Copyright 2004 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.ws.eventing.porttype.impl;

import org.apache.ws.eventing.Subscription;
import org.apache.ws.eventing.porttype.SubscriptionManagerPortType;
import org.apache.ws.resource.NamespaceVersionHolder;
import org.apache.ws.resource.ResourceContext;
import org.apache.ws.resource.properties.impl.AbstractResourcePropertiesPortType;
import org.xmlsoap.schemas.ws.x2004.x08.eventing.GetStatusDocument;
import org.xmlsoap.schemas.ws.x2004.x08.eventing.GetStatusResponseDocument;
import org.xmlsoap.schemas.ws.x2004.x08.eventing.RenewDocument;
import org.xmlsoap.schemas.ws.x2004.x08.eventing.RenewResponseDocument;
import org.xmlsoap.schemas.ws.x2004.x08.eventing.UnsubscribeDocument;
import java.rmi.RemoteException;

/**
 * Impl of Ws-Eventing SubscriptionManager portType.
 *
 * @version $Revision: 1.8 $
 * @author $author$
 */
public class SubscriptionManagerPortTypeImpl
   extends AbstractResourcePropertiesPortType
   implements SubscriptionManagerPortType
{
   /**
    * Creates a new {@link SubscriptionManagerPortTypeImpl} object.
    *
    * @param resourceContext DOCUMENT_ME
    */
   public SubscriptionManagerPortTypeImpl( ResourceContext resourceContext )
   {
      super( resourceContext );
   }

   /**
    * DOCUMENT_ME
    *
    * @param body DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    *
    * @throws RemoteException DOCUMENT_ME
    */
   public GetStatusResponseDocument getStatus( GetStatusDocument body )
   throws RemoteException
   {
      Subscription subscription = (Subscription) getResource(  );

      //build response
      GetStatusResponseDocument                   responseDom = GetStatusResponseDocument.Factory.newInstance(  );
      GetStatusResponseDocument.GetStatusResponse response = responseDom.addNewGetStatusResponse(  );

      //TODO check calendar
      response.setExpires( subscription.getTerminationTime(  ) );
      return ( responseDom );
   }

   /**
    * DOCUMENT_ME
    *
    * @param body DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    *
    * @throws RemoteException DOCUMENT_ME
    */
   public RenewResponseDocument renew( RenewDocument body )
   throws RemoteException
   {
      Subscription subscription = (Subscription) getResource(  );

      //TODO why no ExpirationType
      org.apache.xmlbeans.XmlCalendar et = (org.apache.xmlbeans.XmlCalendar) body.getRenew(  ).getExpires(  );
      subscription.setTerminationTime( et );
      RenewResponseDocument               responseDom = RenewResponseDocument.Factory.newInstance(  );
      RenewResponseDocument.RenewResponse response = responseDom.addNewRenewResponse(  );

      //TODO response with new expirationType
      response.setExpires( subscription.getTerminationTime(  ) );

      return ( responseDom );
   }

   /**
    * DOCUMENT_ME
    *
    * @param body DOCUMENT_ME
    *
    * @throws RemoteException DOCUMENT_ME
    */
   public void unsubscribe( UnsubscribeDocument body )
   throws RemoteException
   {
      Subscription subscription = (Subscription) getResource(  );
      subscription.unsubscribe(  );
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   protected NamespaceVersionHolder getNamespaceSet(  )
   {
      return null; // not currently used - will be needed once we need to support multiple spec versions
   }
}