/*=============================================================================*
 *  Copyright 2004 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.ws.eventing.pubsub;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.ws.notification.base.v2004_06.impl.WsnNamespaceVersionHolderImpl;
import org.apache.ws.resource.IllegalResourceTypeException;
import org.apache.ws.resource.JndiConstants;
import org.apache.ws.resource.NamespaceVersionHolder;
import org.apache.ws.resource.impl.AbstractResourceHome;
import javax.xml.namespace.QName;
import java.util.Map;

/**
 *
 * @author  Stefan Lischke
 */
public class SubscriptionHome
   extends AbstractResourceHome
{
   /**  The service endpoint name as registered with the SOAP Platform.  This is useful for building EPR's. **/
   public static final javax.xml.namespace.QName SERVICE_NAME =
      javax.xml.namespace.QName.valueOf( "{http://ws.apache.org/notification/base/service/SubscriptionEndPort}SubscriptionEndPort" );

   /** The management PortType associated with this resource. This is useful for building EPR's.**/
   public static final javax.xml.namespace.QName PORT_TYPE =
      javax.xml.namespace.QName.valueOf( "http://ws.apache.org/notification/base/service/SubscriptionEndPort}SubscriptionEndPortPortType" );

   /** The WSDL Port name associated with the resource. This is useful for building EPR's. **/
   public static final String PORT_NAME = "SubscriptionEndPort";

   /** The name of the resource key for this resource. **/
   public static final javax.xml.namespace.QName RESOURCE_KEY_NAME =
      javax.xml.namespace.QName.valueOf( "{http://ws.apache.org/notification/base/service/SubscriptionEndPort}ResourceID" );

   //private static Map m_resources=new java.util.HashMap(); 

   /** DOCUMENT_ME */
   public static final String HOME_LOCATION =
      JndiConstants.CONTEXT_NAME_SERVICES + "/" + SERVICE_NAME.getLocalPart(  ) + "/"
      + JndiConstants.ATOMIC_NAME_HOME;
   private static int         num                                 = 0;
   private static final Log   LOG                                 =
      LogFactory.getLog( SubscriptionHome.class.getName(  ) );
   private static final NamespaceVersionHolder SPEC_NAMESPACE_SET = new WsnNamespaceVersionHolderImpl(  );

   /**
    * Map containing all FilesystemResource instances - this map <em>must</em> be static for
    * compatibility with certain JNDI providers.
    */
   private static Map s_resources;

   /**
    * Creates a new {@link SubscriptionHome} object.
    */
   public SubscriptionHome(  )
   {
      num++;
      LOG.info( "create SubscriptionHome number : " + num );
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public NamespaceVersionHolder getNamespaceSet(  )
   {
      return SPEC_NAMESPACE_SET;
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public QName getPortType(  )
   {
      return PORT_TYPE;
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public QName getResourceKeyNameQName(  )
   {
      return RESOURCE_KEY_NAME;
   }

   /*
      public org.apache.ws.resource.Resource find(org.apache.ws.resource.ResourceKey key) throws org.apache.ws.resource.ResourceException, org.apache.ws.resource.NoSuchResourceException, org.apache.ws.resource.InvalidResourceKeyException {
          return((org.apache.ws.resource.Resource)m_resources.get(key));
      }
    */
   /*
      public String getResourceClassName() {
      }
   
      public String getResourceKeyClassName() {
      }
   
      public String getResourceKeyName() {
      }
   
      public String getServiceClassName() {
      }
   
      public String getWsdlTargetNamespace() {
      }*/
   /*
      public void setResourceClassName(String className) {
      }
   
      public void setResourceKeyClassName(String className) {
      }
   
      public void setResourceKeyName(String keyName) {
      }
   
      public void setServiceClassName(String className) {
      }
   
      public void setWsdlTargetNamespace(String targetNamespace) {
      }
    */
   public QName getServiceName(  )
   {
      return SERVICE_NAME;
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public String getServicePortName(  )
   {
      return PORT_NAME;
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public Subscription create(  )
   {
      Subscription s = new Subscription(  );
      s.setResourceHome( this );
      LOG.info( "before add " + m_resources.size(  ) + " instance " + this );
      try
      {
         add( s );
      }
      catch ( IllegalResourceTypeException irte )
      {
         throw new RuntimeException( irte );
      }

      LOG.info( "after add " + m_resources.size(  ) );
      return s;
   }

   /**
    * Returns a map of all FilesystemResource instances. Used by the {@link org.apache.ws.resource.impl.AbstractResourceHome}
    * superclass.
    */
   protected final synchronized Map getResourceMap(  )
   {
      if ( s_resources == null )
      {
         s_resources = AbstractResourceHome.createResourceMap( m_resourceIsPersistent );
      }

      return s_resources;
   }
}