/*=============================================================================*
 *  Copyright 2004 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.ws.notification.base.v2004_06.porttype.impl;

import org.apache.ws.Soap1_1Constants;
import org.apache.ws.XmlObjectWrapper;
import org.apache.ws.addressing.EndpointReference;
import org.apache.ws.addressing.XmlBeansEndpointReference;
import org.apache.ws.notification.base.NotificationProducerResource;
import org.apache.ws.notification.base.WsnNamespaceVersionHolder;
import org.apache.ws.notification.base.faults.InvalidTopicExpressionFaultException;
import org.apache.ws.notification.base.faults.NoCurrentMessageOnTopicFaultException;
import org.apache.ws.notification.base.faults.SubscribeCreationFailedFaultException;
import org.apache.ws.notification.base.faults.TopicNotSupportedFaultException;
import org.apache.ws.notification.base.faults.TopicPathDialectUnknownFaultException;
import org.apache.ws.notification.base.impl.NoCurrentMessageOnTopicException;
import org.apache.ws.notification.base.impl.SubscribeCreationFailedException;
import org.apache.ws.notification.base.impl.TopicNotSupportedException;
import org.apache.ws.notification.base.v2004_06.porttype.NotificationProducerPortType;
import org.apache.ws.notification.topics.TopicsTypeReader;
import org.apache.ws.notification.topics.expression.InvalidTopicExpressionException;
import org.apache.ws.notification.topics.expression.TopicExpression;
import org.apache.ws.notification.topics.expression.TopicPathDialectUnknownException;
import org.apache.ws.resource.ResourceContext;
import org.apache.ws.resource.faults.FaultException;
import org.apache.ws.resource.properties.impl.AbstractResourcePropertiesPortType;
import org.apache.ws.resource.properties.query.QueryExpression;
import org.apache.ws.resource.properties.query.impl.XmlBeansQueryExpression;
import org.apache.ws.util.XmlBeanUtils;
import org.apache.xmlbeans.XmlObject;
import org.oasisOpen.docs.wsn.x2004.x06.wsnWSBaseNotification12Draft01.GetCurrentMessageDocument;
import org.oasisOpen.docs.wsn.x2004.x06.wsnWSBaseNotification12Draft01.GetCurrentMessageResponseDocument;
import org.oasisOpen.docs.wsn.x2004.x06.wsnWSBaseNotification12Draft01.SubscribeDocument;
import org.oasisOpen.docs.wsn.x2004.x06.wsnWSBaseNotification12Draft01.SubscribeResponseDocument;
import org.oasisOpen.docs.wsn.x2004.x06.wsnWSBaseNotification12Draft01.TopicExpressionType;
import org.xmlsoap.schemas.ws.x2003.x03.addressing.EndpointReferenceType;
import java.util.Calendar;

/**
 * An implementation of the WS-BaseNotification NotificationProducer portType.
 *
 * @author Ian Springer
 */
public class NotificationProducerPortTypeImpl
   extends AbstractResourcePropertiesPortType
   implements NotificationProducerPortType
{
   private NotificationProducerResource m_producerResource;

   /**
    * Creates a new {@link NotificationProducerPortTypeImpl} object.
    *
    * @param resourceContext DOCUMENT_ME
    */
   public NotificationProducerPortTypeImpl( ResourceContext resourceContext )
   {
      super( resourceContext );
      m_producerResource = (NotificationProducerResource) getResource(  );
   }

   /**
    * @see NotificationProducerPortType#getCurrentMessage(org.oasisOpen.docs.wsn.x2004.x06.wsnWSBaseNotification12Draft01.GetCurrentMessageDocument)
    */
   public GetCurrentMessageResponseDocument getCurrentMessage( GetCurrentMessageDocument requestDoc )
   {
      GetCurrentMessageDocument.GetCurrentMessage request    = requestDoc.getGetCurrentMessage(  );
      TopicExpression                             topicExpr  = toTopicExpression( request.getTopic(  ) );
      Object                                      currentMsg = null;
      try
      {
         currentMsg = m_producerResource.getCurrentMessage( topicExpr );
      }
      catch ( InvalidTopicExpressionException itee )
      {
         throw new InvalidTopicExpressionFaultException( getNamespaceSet(  ),
                                                         itee.getLocalizedMessage(  ) );
      }
      catch ( TopicNotSupportedException tnse )
      {
         throw new TopicNotSupportedFaultException( getNamespaceSet(  ),
                                                    tnse.getLocalizedMessage(  ) );
      }
      catch ( NoCurrentMessageOnTopicException ncmote )
      {
         throw new NoCurrentMessageOnTopicFaultException( getNamespaceSet(  ),
                                                          ncmote.getLocalizedMessage(  ) );
      }

      return buildGetCurrentMessageResponseDocument( currentMsg );
   }

   /**
    * @see NotificationProducerPortType#subscribe(org.oasisOpen.docs.wsn.x2004.x06.wsnWSBaseNotification12Draft01.SubscribeDocument)
    */
   public SubscribeResponseDocument subscribe( SubscribeDocument requestDoc )
   {
      SubscribeDocument.Subscribe request                = requestDoc.getSubscribe(  );
      TopicExpression             topicExpr = null;
      topicExpr = toTopicExpression( request.getTopicExpression(  ) );

      Calendar initialTerminationTime = null;
      if ( request.isSetInitialTerminationTime(  ) )
      {
         initialTerminationTime = request.getInitialTerminationTime(  );
      }

      QueryExpression precondition = null;
      if ( request.isSetPrecondition(  ) )
      {
         precondition = new XmlBeansQueryExpression( request.getPrecondition(  ) );
      }

      QueryExpression selector = null;
      if ( request.isSetSelector(  ) )
      {
         selector = new XmlBeansQueryExpression( request.getSelector(  ) );
      }

      XmlObject subPolicy = null;
      if ( request.isSetSubscriptionPolicy(  ) )
      {
         subPolicy = request.getSubscriptionPolicy(  );
      }

      Boolean           useNotify =
         request.isSetUseNotify(  ) ? Boolean.valueOf( request.getUseNotify(  ) ) : null;
      EndpointReference consumerEPR     = new XmlBeansEndpointReference( request.getConsumerReference(  ) );
      EndpointReference subscriptionEPR;
      try
      {
         subscriptionEPR =
            m_producerResource.subscribe( consumerEPR, topicExpr, useNotify, precondition, selector, subPolicy,
                                          initialTerminationTime );
      }
      catch ( SubscribeCreationFailedException scfe )
      {
         throw new SubscribeCreationFailedFaultException( getNamespaceSet(  ) );
      }
      catch ( InvalidTopicExpressionException itee )
      {
         throw new InvalidTopicExpressionFaultException( getNamespaceSet(  ),
                                                         itee.getLocalizedMessage(  ) );
      }
      catch ( TopicPathDialectUnknownException tpdue )
      {
         throw new TopicPathDialectUnknownFaultException( getNamespaceSet(  ),
                                                          tpdue.getLocalizedMessage(  ) );
      }

      return buildSubscribeResponseDocument( subscriptionEPR );
   }

   private WsnNamespaceVersionHolder getNamespaceSet(  )
   {
      return (WsnNamespaceVersionHolder) getResource(  ).getNamespaceSet(  );
   }

   private GetCurrentMessageResponseDocument buildGetCurrentMessageResponseDocument( Object currentMsg )
   {
      XmlObject msgXBean;
      try
      {
         msgXBean = XmlBeanUtils.toXmlObject( currentMsg );
      }
      catch ( Exception e )
      {
         throw new FaultException( Soap1_1Constants.FAULT_SERVER, "Internal server error." );
      }

      GetCurrentMessageResponseDocument                           responseDoc =
         GetCurrentMessageResponseDocument.Factory.newInstance(  );
      GetCurrentMessageResponseDocument.GetCurrentMessageResponse response =
         responseDoc.addNewGetCurrentMessageResponse(  );
      XmlBeanUtils.addChildElement( response, msgXBean );
      return responseDoc;
   }

   private SubscribeResponseDocument buildSubscribeResponseDocument( EndpointReference subscriptionEPR )
   {
      EndpointReferenceType                       subscriptionRefType;
      SubscribeResponseDocument                   subscribeResponseDoc =
         SubscribeResponseDocument.Factory.newInstance(  );
      SubscribeResponseDocument.SubscribeResponse subscribeResponse =
         subscribeResponseDoc.addNewSubscribeResponse(  );
      subscriptionRefType = (EndpointReferenceType) ( (XmlObjectWrapper) subscriptionEPR ).getXmlObject(  );
      subscribeResponse.setSubscriptionReference( subscriptionRefType );
      return subscribeResponseDoc;
   }

   private TopicExpression toTopicExpression( TopicExpressionType topicExprType )
   {
      TopicsTypeReader topicsTypeReader =
         TopicsTypeReader.newInstance( getNamespaceSet(  ).getTopicsXsdNamespace(  ) );
      try
      {
         return topicsTypeReader.toTopicExpression( topicExprType );
      }
      catch ( InvalidTopicExpressionException itee )
      {
         throw new InvalidTopicExpressionFaultException( getNamespaceSet(  ),
                                                         itee.getLocalizedMessage(  ) );
      }
   }
}