/*=============================================================================*
 *  Copyright 2004 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.ws.notification.base.v2004_06.porttype.impl;

import org.apache.ws.notification.base.PauseFailedException;
import org.apache.ws.notification.base.ResumeFailedException;
import org.apache.ws.notification.base.Subscription;
import org.apache.ws.notification.base.WsnNamespaceVersionHolder;
import org.apache.ws.notification.base.faults.PauseFailedFaultException;
import org.apache.ws.notification.base.faults.ResumeFailedFaultException;
import org.apache.ws.notification.base.v2004_06.impl.WsnNamespaceVersionHolderImpl;
import org.apache.ws.notification.base.v2004_06.porttype.SubscriptionManagerPortType;
import org.apache.ws.resource.NamespaceVersionHolder;
import org.apache.ws.resource.ResourceContext;
import org.apache.ws.resource.properties.impl.AbstractResourcePropertiesPortType;
import org.oasisOpen.docs.wsn.x2004.x06.wsnWSBaseNotification12Draft01.PauseSubscriptionDocument;
import org.oasisOpen.docs.wsn.x2004.x06.wsnWSBaseNotification12Draft01.PauseSubscriptionResponseDocument;
import org.oasisOpen.docs.wsn.x2004.x06.wsnWSBaseNotification12Draft01.ResumeSubscriptionDocument;
import org.oasisOpen.docs.wsn.x2004.x06.wsnWSBaseNotification12Draft01.ResumeSubscriptionResponseDocument;

/**
 * An implementation of the WS-BaseNotification 2004/06 SubscriptionManager portType.
 *
 * @author Ian Springer
 */
public class SubscriptionManagerPortTypeImpl
   extends AbstractResourcePropertiesPortType
   implements SubscriptionManagerPortType
{
   private static final WsnNamespaceVersionHolder WSN_2004_06_NAMESPACE_SET =
      new WsnNamespaceVersionHolderImpl(  );

   /**
    * Creates a new {@link SubscriptionManagerPortTypeImpl} object.
    *
    * @param resourceContext DOCUMENT_ME
    */
   public SubscriptionManagerPortTypeImpl( ResourceContext resourceContext )
   {
      super( resourceContext );
   }

   /**
    * DOCUMENT_ME
    *
    * @param requestDoc DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public PauseSubscriptionResponseDocument pauseSubscription( PauseSubscriptionDocument requestDoc )
   {
      try
      {
         getSubscription(  ).pause(  );
      }
      catch ( PauseFailedException pfe )
      {
         throw new PauseFailedFaultException( (WsnNamespaceVersionHolder) getResource(  ).getNamespaceSet(  ),
                                              "Pause failed on subscription with id "
                                              + getSubscription(  ).getID(  ) );
      }

      return createPauseSubscriptionResponseDocument(  );
   }

   /**
    * DOCUMENT_ME
    *
    * @param requestDoc DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public ResumeSubscriptionResponseDocument resumeSubscription( ResumeSubscriptionDocument requestDoc )
   {
      try
      {
         getSubscription(  ).resume(  );
      }
      catch ( ResumeFailedException rfe )
      {
         throw new ResumeFailedFaultException( (WsnNamespaceVersionHolder) getResource(  ).getNamespaceSet(  ),
                                               "Resume failed on subscription with id "
                                               + getSubscription(  ).getID(  ) );
      }

      return createResumeSubscriptionResponseDocument(  );
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   protected NamespaceVersionHolder getNamespaceSet(  )
   {
      return WSN_2004_06_NAMESPACE_SET;
   }

   private Subscription getSubscription(  )
   {
      return (Subscription) getResource(  );
   }

   private PauseSubscriptionResponseDocument createPauseSubscriptionResponseDocument(  )
   {
      PauseSubscriptionResponseDocument responseDoc = PauseSubscriptionResponseDocument.Factory.newInstance(  );
      responseDoc.addNewPauseSubscriptionResponse(  );
      return responseDoc;
   }

   private ResumeSubscriptionResponseDocument createResumeSubscriptionResponseDocument(  )
   {
      ResumeSubscriptionResponseDocument responseDoc = ResumeSubscriptionResponseDocument.Factory.newInstance(  );
      responseDoc.addNewResumeSubscriptionResponse(  );
      return responseDoc;
   }
}