/*=============================================================================*
 *  Copyright 2004 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.ws.notification.pubsub;

import org.apache.ws.resource.IllegalResourceTypeException;
import org.apache.ws.resource.JndiConstants;
import org.apache.ws.resource.NamespaceVersionHolder;
import org.apache.ws.resource.impl.AbstractResourceHome;
import javax.xml.namespace.QName;
import java.util.Map;

/**
 *
 * @author  Stefan Lischke
 */
public class SubscriptionHome
   extends AbstractResourceHome
{
   /**  The service endpoint name as registered with the SOAP Platform.  This is useful for building EPR's. **/
   public static final javax.xml.namespace.QName SERVICE_NAME =
      javax.xml.namespace.QName.valueOf( "{http://ws.apache.org/notification/base/service/NotificationPort}NotificationPort" );

   /** The management PortType associated with this resource. This is useful for building EPR's.**/
   public static final javax.xml.namespace.QName PORT_TYPE =
      javax.xml.namespace.QName.valueOf( "http://ws.apache.org/notification/base/service/NotificationPort}NotificationPortPortType" );

   /** The WSDL Port name associated with the resource. This is useful for building EPR's. **/
   public static final String PORT_NAME = "NotificationPort";

   /** The name of the resource key for this resource. **/
   public static final javax.xml.namespace.QName RESOURCE_KEY_NAME =
      javax.xml.namespace.QName.valueOf( "{http://ws.apache.org/notification/base/service/NotificationPort}ResourceID" );

   /** DOCUMENT_ME */
   public static final String HOME_LOCATION =
      JndiConstants.CONTEXT_NAME_SERVICES + "/" + SERVICE_NAME.getLocalPart(  ) + "/"
      + JndiConstants.ATOMIC_NAME_HOME;

   /**
    * Map containing all FilesystemResource instances - this map <em>must</em> be static for
    * compatibility with certain JNDI providers.
    */
   private static Map s_resources;

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public NamespaceVersionHolder getNamespaceSet(  )
   {
      return null; //not sure what should go here.
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public QName getPortType(  )
   {
      return PORT_TYPE;
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public QName getResourceKeyNameQName(  )
   {
      return RESOURCE_KEY_NAME;
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public QName getServiceName(  )
   {
      return SERVICE_NAME;
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public String getServicePortName(  )
   {
      return PORT_NAME;
   }

   //private static final Log LOG = LogFactory.getLog( SubscriptionHome.class.getName() );
   public Subscription create(  )
   {
      Subscription s = new Subscription( null, null, null );
      try
      {
         add( s );
      }
      catch ( IllegalResourceTypeException irte )
      {
         throw new RuntimeException( irte );
      }

      return s;
   }

   /**
    * Returns a map of all SubscriptionResource instances. Used by the {@link AbstractResourceHome}
    * superclass.
    */
   protected final synchronized Map getResourceMap(  )
   {
      if ( s_resources == null )
      {
         s_resources = AbstractResourceHome.createResourceMap( m_resourceIsPersistent );
      }

      return s_resources;
   }
}