/*=============================================================================*
 *  Copyright 2004 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.ws.notification.topics;


/**
 * A topic namespace (a hierarchy of topics within a particular namespace), as defined by the WS-Topics 1.3
 * specification.
 */
public interface TopicNamespace
   extends TopicSiblingSet
{
   /**
    * Sets this topic namespace's finality.
    *
    * @param finality the finality
    */
   void setFinal( boolean finality );

   /**
    * If the value is <code>true</code> it indicates that NotificationProducers cannot use topics whose root topics do
    * not appear in this topic namespace. The default value is <code>false</code>.
    *
    * @return true if this topic namespace is final, or false if it is not
    */
   boolean isFinal(  );

   /**
    * Sets this topic namespace's (optional) name.
    *
    * @param name the name
    */
   void setName( String name );

   /**
    * Returns this topic namespace's name, or null if no name is defined.
    *
    * @return this topic namespace's name, or null if no name is defined
    */
   String getName(  );

   /**
    * Returns this topic namespace's target namespace.
    *
    * @return this topic namespace's target namespace
    */
   String getTargetNamespace(  );
}