/*=============================================================================*
 *  Copyright 2004 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.ws.notification.topics;

import org.apache.ws.notification.topics.expression.TopicExpression;
import org.apache.ws.notification.topics.expression.TopicExpressionException;
import javax.xml.namespace.QName;

/**
 * The set of topics that a particular NotificationProducer supports.
 */
public interface TopicSet
{
   /**
    * Sets the boolean that indicates if this topic set is fixed as defined by the WS-BaseNotification specification.
    *
    * @param fixed
    */
   void setFixed( boolean fixed );

   /**
    * Returns a boolean that indicates if this topic set is fixed as defined by the WS-BaseNotification specification.
    */
   boolean isFixed(  );

   /**
    * Returns the topic expressions contained in this topic set.
    */
   TopicExpression[] getTopicExpressions(  );

   /**
    * Returns the topics contained in this topic set.
    */
   Topic[] getTopics(  );

   /**
    * @param topicExpr
    */
   void addTopicExpression( TopicExpression topicExpr )
   throws TopicExpressionException;

   /**
    *
    * @param topicPath the path to exactly one Topic (e.g. new QName( "http://example.org/topicSpace/example1", "t4/t5" ))
    *
    * @return true if this topic set contains the specified topic, or false if it does not
    */
   boolean containsTopic( QName topicPath );

   /**
    *
    * @param topicExpr
    * @return
    * @throws TopicExpressionException
    */
   Topic[] evaluateTopicExpression( TopicExpression topicExpr )
   throws TopicExpressionException;
}