/*=============================================================================*
 *  Copyright 2004 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.ws.notification.topics.expression;

import org.apache.ws.notification.topics.expression.impl.AbstractTopicExpression;
import org.apache.ws.notification.topics.v2004_06.TopicsConstants;
import javax.xml.namespace.QName;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.OptionalDataException;
import java.util.ArrayList;

/**
 * A Full TopicExpression as defined by the WS-Topics specification.
 */
public class FullTopicExpression
   extends AbstractTopicExpression
{
   private transient QName[] m_topicPaths;

   /**
    * Creates a new {@link FullTopicExpression} object.
    *
    * @param topicPaths DOCUMENT_ME
    */
   public FullTopicExpression( QName[] topicPaths )
   {
      super( TopicsConstants.TOPIC_EXPR_DIALECT_FULL, topicPaths );
      m_topicPaths = topicPaths;
   }

   /**
    * Creates a new {@link FullTopicExpression} object.
    */
   protected FullTopicExpression(  )
   {
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public QName[] getTopicPaths(  )
   {
      return m_topicPaths;
   }

   /**
    * DOCUMENT_ME
    *
    * @param obj DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public boolean equals( Object obj )
   {
      return ( obj != null ) && obj instanceof FullTopicExpression && ( obj.hashCode(  ) == hashCode(  ) );
   }

   private void readObject( ObjectInputStream in )
   throws IOException, 
          ClassNotFoundException
   {
      ArrayList list = new ArrayList(  );
      while ( true ) //loop the input stream to read in the array
      {
         try
         {
            String qname = (String) in.readObject(  );
            list.add( QName.valueOf( qname ) );
         }
         catch ( OptionalDataException e ) //catch this exception to determine eof
         {
            if ( e.eof == true )
            {
               break;
            }
            else
            {
               throw e;
            }
         }
      }

      m_topicPaths = (QName[]) list.toArray( new QName[0] );
      setContent( m_topicPaths );
      setDialect( TopicsConstants.TOPIC_EXPR_DIALECT_FULL );
   }

   private void writeObject( ObjectOutputStream out )
   throws IOException
   {
      for ( int i = 0; i < m_topicPaths.length; i++ )
      {
         out.writeObject( m_topicPaths[i].toString(  ) );
      }
   }
}