/*=============================================================================*
 *  Copyright 2004 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.ws.notification.topics.util;

import org.apache.ws.notification.base.NotificationProducerResource;
import org.apache.ws.notification.topics.impl.ResourcePropertyValueChangeListenerImpl;
import org.apache.ws.notification.topics.impl.ResourceTerminationListenerImpl;
import org.apache.ws.resource.lifetime.v2004_06.ResourceLifetimeConstants;
import org.apache.ws.resource.properties.ResourceProperty;
import java.util.Iterator;

/**
 * A utility class containing static methods used as helpers for working with WSN Topics.
 *
 * @author Ian Springer
 */
public class TopicUtils
{
   /**
    * Adds topics from the specified notification producer resource's property set to
    * its topic set.
    *
    * @param producerResource
    */
   public static void addResourcePropertyValueChangeListeners( NotificationProducerResource producerResource )
   throws Exception
   {
      Iterator propIter = producerResource.getResourcePropertySet(  ).iterator(  );
      while ( propIter.hasNext(  ) )
      {
         ResourceProperty prop = (ResourceProperty) propIter.next(  );
         if ( producerResource.getTopicSet(  ).containsTopic( prop.getMetaData(  ).getName(  ) ) )
         {
            ResourcePropertyValueChangeListenerImpl propValueChangeListener =
               new ResourcePropertyValueChangeListenerImpl( producerResource, prop );
            prop.addChangeListener( propValueChangeListener );
            producerResource.getTopicSet(  ).addTopicExpression( propValueChangeListener.getTopicExpression(  ) );
         }
      }
   }

   /**
    * Configures the specified producer resource to support the WSRF ResourceTermination topic.
    *
    * @param producerResource
    */
   public static void addResourceTerminationListener( NotificationProducerResource producerResource )
   throws Exception
   {
      if ( producerResource.getTopicSet(  ).containsTopic( ResourceLifetimeConstants.TOPIC_PATH_RESOURCE_TERMINATION ) )
      {
         ResourceTerminationListenerImpl resourceTerminationListener =
            new ResourceTerminationListenerImpl( producerResource );
         producerResource.addTerminationListener( resourceTerminationListener );
      }
   }
}