/*=============================================================================*
 *  Copyright 2004 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.ws.notification.base.impl;

import org.apache.ws.addressing.EndpointReference;
import org.apache.ws.notification.base.NotificationProducerResource;
import org.apache.ws.notification.topics.expression.InvalidTopicExpressionException;
import org.apache.ws.notification.topics.expression.TopicExpression;
import org.apache.ws.notification.topics.expression.TopicPathDialectUnknownException;
import org.apache.ws.resource.NamespaceVersionHolder;
import org.apache.ws.resource.PropertiesResource;
import org.apache.ws.resource.lifetime.ResourceTerminationListener;
import org.apache.ws.resource.properties.ResourcePropertySet;
import org.apache.ws.resource.properties.impl.XmlBeansResourcePropertySet;
import org.apache.ws.resource.properties.query.QueryExpression;
import org.apache.ws.resource.properties.v2004_06.impl.NamespaceVersionHolderImpl;
import org.oasisOpen.docs.wsn.x2004.x06.wsnWSBaseNotification12Draft01.NotificationProducerRPDocument;
import java.util.Calendar;

/**
 * @author Sal Campana
 */
public class UnitResource
   implements PropertiesResource,
              NotificationProducerResource
{
   /** DOCUMENT_ME */
   public static final String TOPIC_NAME = "test";

   /** DOCUMENT_ME */
   ResourcePropertySet       m_resourcePropertySet;
   private EndpointReference m_epr;
   private Object            m_id;

   /** DOCUMENT_ME */
   org.apache.ws.notification.topics.TopicSet m_topicSet =
      new org.apache.ws.notification.topics.impl.TopicSetImpl(  );

   /**
    * DOCUMENT_ME
    *
    * @param topicExpr DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public Object getCurrentMessage( TopicExpression topicExpr )
   {
      return null; //To change body of implemented methods use File | Settings | File Templates.
   }

   /**
    * DOCUMENT_ME
    *
    * @param epr DOCUMENT_ME
    */
   public void setEndpointReference( EndpointReference epr )
   {
      m_epr = epr;
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public EndpointReference getEndpointReference(  )
   {
      return m_epr;
   }

   /**
    * DOCUMENT_ME
    *
    * @param resourceId DOCUMENT_ME
    */
   public void setID( Object resourceId )
   {
      m_id = resourceId;
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public Object getID(  )
   {
      return m_id;
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public NamespaceVersionHolder getNamespaceSet(  )
   {
      return new NamespaceVersionHolderImpl(  );
   }

   /**
    * DOCUMENT_ME
    *
    * @param resourcePropertySet DOCUMENT_ME
    */
   public void setResourcePropertySet( ResourcePropertySet resourcePropertySet )
   {
      m_resourcePropertySet = resourcePropertySet;
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public ResourcePropertySet getResourcePropertySet(  )
   {
      return m_resourcePropertySet;
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public org.apache.ws.notification.topics.TopicSet getTopicSet(  )
   {
      return m_topicSet;
   }

   /**
    * DOCUMENT_ME
    *
    * @param resourceTerminationListener DOCUMENT_ME
    */
   public void addTerminationListener( ResourceTerminationListener resourceTerminationListener )
   {
   }

   /**
    * DOCUMENT_ME
    */
   public void destroy(  )
   {
   }

   /**
    * DOCUMENT_ME
    */
   public void init(  )
   {
      NotificationProducerRPDocument notificationProducerRPDocument =
         NotificationProducerRPDocument.Factory.newInstance(  );
      notificationProducerRPDocument.addNewNotificationProducerRP(  );
      m_resourcePropertySet = new XmlBeansResourcePropertySet( notificationProducerRPDocument );

      org.apache.ws.resource.properties.ResourceProperty topicProp =
         getResourcePropertySet(  ).get( org.apache.ws.notification.base.v2004_06.porttype.NotificationProducerPortType.PROP_QNAME_TOPIC );
      topicProp.setCallback( new org.apache.ws.notification.base.TopicResourcePropertyCallback( getTopicSet(  ) ) );

      org.apache.ws.resource.properties.ResourceProperty fixedTopicSetProp =
         getResourcePropertySet(  ).get( org.apache.ws.notification.base.v2004_06.porttype.NotificationProducerPortType.PROP_QNAME_FIXED_TOPIC_SET );
      fixedTopicSetProp.setCallback( new org.apache.ws.notification.base.FixedTopicSetResourcePropertyCallback( getTopicSet(  ) ) );

      org.apache.ws.resource.properties.ResourceProperty topicExpressionDialectsProp =
         getResourcePropertySet(  ).get( org.apache.ws.notification.base.v2004_06.porttype.NotificationProducerPortType.PROP_QNAME_TOPIC_EXPRESSION_DIALECTS );
      topicExpressionDialectsProp.setCallback( new org.apache.ws.notification.base.TopicExpressionDialectsResourcePropertyCallback(  ) );
      try
      {
         org.apache.ws.notification.topics.util.TopicUtils.addResourcePropertyValueChangeListeners( this );
      }
      catch ( Exception e )
      {
         throw new javax.xml.rpc.JAXRPCException( "Unable to init the ResourceProperty Changed topics. Cause: "
                                                  + e.getLocalizedMessage(  ), e );
      }
   }

   /**
    * DOCUMENT_ME
    *
    * @param topic DOCUMENT_ME
    * @param msg DOCUMENT_ME
    */
   public void publish( TopicExpression topic,
                        Object          msg )
   {
      // TODO
   }

   /**
    * DOCUMENT_ME
    *
    * @param consumerEPR DOCUMENT_ME
    * @param topicExpr DOCUMENT_ME
    * @param useNotify DOCUMENT_ME
    * @param precondition DOCUMENT_ME
    * @param selector DOCUMENT_ME
    * @param subscriptionPolicy DOCUMENT_ME
    * @param initialTerminationtime DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    *
    * @throws SubscribeCreationFailedException DOCUMENT_ME
    * @throws InvalidTopicExpressionException DOCUMENT_ME
    * @throws TopicPathDialectUnknownException DOCUMENT_ME
    */
   public EndpointReference subscribe( EndpointReference consumerEPR,
                                       TopicExpression   topicExpr,
                                       Boolean           useNotify,
                                       QueryExpression   precondition,
                                       QueryExpression   selector,
                                       Object            subscriptionPolicy,
                                       Calendar          initialTerminationtime )
   throws SubscribeCreationFailedException, 
          InvalidTopicExpressionException, 
          TopicPathDialectUnknownException
   {
      return null; //To change body of implemented methods use File | Settings | File Templates.
   }
}