/*=============================================================================*
 *  Copyright 2004 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.ws.notification.topics.impl;

import junit.framework.TestCase;
import org.apache.ws.notification.topics.Topic;
import org.apache.ws.notification.topics.TopicNamespace;
import org.apache.ws.notification.topics.TopicNamespaceRegistry;
import org.apache.ws.notification.topics.TopicsTypeReader;
import org.apache.ws.notification.topics.expression.InvalidTopicExpressionException;
import org.apache.ws.notification.topics.expression.TopicExpressionEvaluator;
import org.apache.ws.notification.topics.v2004_06.TopicsConstants;
import org.apache.xmlbeans.XmlObject;
import java.net.URI;

/**
 * Abstract test case for various {@link TopicExpressionEvaluator} impls.
 */
public abstract class AbstractTopicExpressionEvaluatorTestCase
   extends TestCase
{
   /** DOCUMENT_ME */
   protected static final URI         NSURI1            = URI.create( "http://ns1.com/" );

   /** DOCUMENT_ME */
   protected static final String      NSPREFIX1         = "ns1";

   /** DOCUMENT_ME */
   protected static final URI         NSURI2            = URI.create( "http://ns2.com/" );

   /** DOCUMENT_ME */
   protected static final String      NSPREFIX2         = "ns2";

   /** DOCUMENT_ME */
   protected TopicExpressionEvaluator m_evaluator;

   /** DOCUMENT_ME */
   protected TopicNamespaceRegistry m_topicNsRegistry;

   /**
    * DOCUMENT_ME
    *
    * @throws Exception DOCUMENT_ME
    */
   protected void setUp(  )
   throws Exception
   {
      m_topicNsRegistry = new TopicNamespaceRegistry(  );

      XmlObject      topicNsXBean =
         XmlObject.Factory.parse( this.getClass(  ).getClassLoader(  ).getResourceAsStream( "org/apache/ws/notification/test-topicSpace.xml" ) );
      TopicNamespace topicNs =
         TopicsTypeReader.newInstance( TopicsConstants.NSURI_WSTOP_SCHEMA ).toTopicNamespace( topicNsXBean );
      m_topicNsRegistry.addTopicNamespace( topicNs );

      topicNsXBean =
         XmlObject.Factory.parse( this.getClass(  ).getClassLoader(  ).getResourceAsStream( "org/apache/ws/notification/test2-topicSpace.xml" ) );
      topicNs =
         TopicsTypeReader.newInstance( TopicsConstants.NSURI_WSTOP_SCHEMA ).toTopicNamespace( topicNsXBean );
      m_topicNsRegistry.addTopicNamespace( topicNs );

      topicNsXBean =
         XmlObject.Factory.parse( this.getClass(  ).getClassLoader(  ).getResourceAsStream( "org/apache/ws/notification/adHoc-topicSpace.xml" ) );
      topicNs =
         TopicsTypeReader.newInstance( TopicsConstants.NSURI_WSTOP_SCHEMA ).toTopicNamespace( topicNsXBean );
      m_topicNsRegistry.addTopicNamespace( topicNs );

      m_evaluator = createTopicExpressionEvaluator(  );
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   protected abstract TopicExpressionEvaluator createTopicExpressionEvaluator(  );

   /**
    * DOCUMENT_ME
    *
    * @param topics DOCUMENT_ME
    * @param name DOCUMENT_ME
    */
   protected void assertContainsTopic( Topic[] topics,
                                       String  name )
   {
      boolean foundIt = false;
      for ( int i = 0; i < topics.length; i++ )
      {
         if ( topics[i].getName(  ).equals( name ) )
         {
            foundIt = true;
         }
      }

      assertTrue( foundIt );
   }

   /**
    * DOCUMENT_ME
    *
    * @param expr DOCUMENT_ME
    *
    * @throws Exception DOCUMENT_ME
    */
   protected void assertExpressionIsInvalid( String expr )
   throws Exception
   {
      try
      {
         evaluate( expr );
         fail(  );
      }
      catch ( InvalidTopicExpressionException itee )
      {
         // success!
      }
   }

   /**
    * DOCUMENT_ME
    *
    * @param expr DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    *
    * @throws Exception DOCUMENT_ME
    */
   protected abstract Topic[] evaluate( String expr )
   throws Exception;
}