/*=============================================================================*
 *  Copyright 2004 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.ws.notification.topics.impl;

import org.apache.ws.notification.topics.Topic;
import org.apache.ws.notification.topics.TopicsTypeReader;
import org.apache.ws.notification.topics.expression.TopicExpression;
import org.apache.ws.notification.topics.expression.TopicExpressionEvaluator;
import org.apache.ws.notification.topics.expression.impl.ConcreteTopicExpressionEvaluator;
import org.apache.ws.notification.topics.v2004_06.TopicsConstants;
import org.apache.xmlbeans.XmlObject;
import org.oasisOpen.docs.wsn.x2004.x06.wsnWSBaseNotification12Draft01.TopicExpressionDocument;
import org.oasisOpen.docs.wsn.x2004.x06.wsnWSBaseNotification12Draft01.TopicExpressionType;

/**
 * Test case for {@link ConcreteTopicExpressionEvaluator}.
 */
public class ConcreteTopicExpressionEvaluatorTestCase
   extends AbstractTopicExpressionEvaluatorTestCase
{
   /**
    * DOCUMENT_ME
    *
    * @throws Exception DOCUMENT_ME
    */
   public void testEvaluateInvalid(  )
   throws Exception
   {
      // check expressions that do not conform to the Concrete Topic Expression grammar defined by the WS-Topics spec
      assertExpressionIsInvalid( NSPREFIX1 + ":" );

      //assertExpressionIsInvalid( ":sports" );
      assertExpressionIsInvalid( NSPREFIX1 + "::sports" );
      assertExpressionIsInvalid( NSPREFIX1 + ":sports:tennis" );
      assertExpressionIsInvalid( NSPREFIX1 + ":sports/*" );
      assertExpressionIsInvalid( NSPREFIX1 + ":sports/." );
      assertExpressionIsInvalid( NSPREFIX1 + ":sports//tennis" );
      assertExpressionIsInvalid( NSPREFIX1 + ":sports|" + NSPREFIX2 + ":bands" );
   }

   /**
    * DOCUMENT_ME
    *
    * @throws Exception DOCUMENT_ME
    */
   public void testEvaluateValid(  )
   throws Exception
   {
      Topic[] resultTopics = evaluate( "candy/nerds" ); // topic from ad-hoc ns
      assertEquals( 1, resultTopics.length );

      resultTopics = evaluate( NSPREFIX1 + ":sports" );
      assertEquals( 1, resultTopics.length );
      assertContainsTopic( resultTopics, "sports" );

      resultTopics = evaluate( NSPREFIX1 + ":bands" );
      assertEquals( 0, resultTopics.length );

      resultTopics = evaluate( NSPREFIX1 + ":sports/tennis" );
      assertEquals( 1, resultTopics.length );
      assertContainsTopic( resultTopics, "tennis" );

      resultTopics = evaluate( NSPREFIX1 + ":sports/baseball" );
      assertEquals( 0, resultTopics.length );
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   protected TopicExpressionEvaluator createTopicExpressionEvaluator(  )
   {
      return new ConcreteTopicExpressionEvaluator(  );
   }

   /**
    * DOCUMENT_ME
    *
    * @param expr DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    *
    * @throws Exception DOCUMENT_ME
    */
   protected Topic[] evaluate( String expr )
   throws Exception
   {
      TopicExpressionType topicExprType =
         ( (TopicExpressionDocument) XmlObject.Factory.parse( "<wsnt:TopicExpression xmlns:wsnt='http://docs.oasis-open.org/wsn/2004/06/wsn-WS-BaseNotification-1.2-draft-01.xsd' "
                                                              + "xmlns:" + NSPREFIX1 + "='" + NSURI1 + "' xmlns:"
                                                              + NSPREFIX2 + "='" + NSURI2
                                                              + "' Dialect='http://docs.oasis-open.org/wsn/2004/06/TopicExpression/Concrete'>"
                                                              + expr + "</wsnt:TopicExpression>" ) )
         .getTopicExpression(  );
      TopicExpression     topicExpr =
         TopicsTypeReader.newInstance( TopicsConstants.NSURI_WSTOP_SCHEMA ).toTopicExpression( topicExprType );
      return m_evaluator.evaluate( m_topicNsRegistry, topicExpr );
   }
}